<?php
namespace Teknisa\PLA\Modules\Cardapio\Controllers;

use Zeedhi\Framework\DTO;
use Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Setforms2\Shared\ORM\DateTime;

class Pla07700AprovacaoCardapioMatriz extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $novoCodigo;
    protected $crudService;
    protected $environment;
    protected $entityManager;
    protected $elaboracaoService;
    protected $aprovacaoCardapioMatrizService;

    public function __construct($crudService, $parameterBag, Manager $dataSourceManager, Environment $environment, $novoCodigo, $entityManager, $elaboracaoService, $aprovacaoCardapioMatrizService) {

        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
    	$this->novoCodigo = $novoCodigo;
    	$this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->elaboracaoService = $elaboracaoService;
    	$this->aprovacaoCardapioMatrizService = $aprovacaoCardapioMatrizService;

    }
    
    
    /**
    * Função resposável por retornar o campo observação geral e os dados do grid
    * para o requerente.
    * 
    * @param {Request} $request Requisição feita pelo usuário.
    * @param {Response} $response Resposta a requisição do usuário.
    */
    public function getFilteredData(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $dtMesCard = '01/'.$row->get('DTMESCARD');
        
        $paramsToExecute = array();
        $nrlista = $row["NRLISTASERVICO"];
        
        $operadorLibMatr = $this->aprovacaoCardapioMatrizService->getOperadorLibMatr();
        $operadorForaMeta = $this->aprovacaoCardapioMatrizService->getOperadorForaMeta();
        $dsgeraprocardmatr = $this->aprovacaoCardapioMatrizService->getDsgeraprocardmatr($dtMesCard);
        
        $response->addDataSet(new DataSet('DSGERAPROCARDMATR', $dsgeraprocardmatr));
        $response->addDataSet(new DataSet('GRPLIBMATRIZ', array($operadorLibMatr))); 
        $response->addDataSet(new DataSet('GRPFORAMETA', array($operadorForaMeta))); 
    }
    
    
    /**
    * Função responsável por aprovar cardápio caso todas as verificações forem
    * atendidas. Aprova cardápio de acordo com a aprovação pela filial.
    * 
    * @param {Request} $request Requisição feita pelo Usuário.
    * @param {Response} $response Resposta a requisição do usuário.
    */
    public function aprovarCard(DTO\Request $request, DTO\Response $response){
        
        $rows = $request->getRow()->get('ROWS')['dataset'];
        $dataToSave = $request->getRow()->get('DATA');
        $responseMessage = array();

        $arrayValorCusto = $this->getVrpuCtrlAprCardMatr($rows, $dataToSave);

        foreach ($rows as $register) {
            
            $valorCustoProdMat = array_filter($arrayValorCusto, function($item) use ($register){
                return ($item['CDFILIAL']==$register['CDFILIAL'] &&
                $item['CDSERVFILI']==$register['CDSERVFILI']);
            });
            
            if(isset($valorCustoProdMat[0]) && 
                isset($valorCustoProdMat[0]['VRCUSTPRODMAT'])){
                    $register['VRCUSTPRODMAT'] = $valorCustoProdMat[0]['VRCUSTPRODMAT'];
            } else {
                $register['VRCUSTPRODMAT'] = 0;
            }
            
            $isRestricted = $this->isRestrictedCard($register, $dataToSave['DTMESCARD']);
            $isAprovedFilial = $this->isAprovedCardFilial($register, $dataToSave['DTMESCARD']);
            $isExpiredDataLimiteAprovacaoFilial = $this->isExpiredDataLimiteAprovacaoFilial($register, '01/'.$dataToSave['DTMESCARD']);
            $isExpiredDataLimiteAprovacaoMatriz = $this->isExpiredDataLimiteAprovacaoMatriz($register, '01/'.$dataToSave['DTMESCARD']);
            

            if(!$isRestricted && !$isExpiredDataLimiteAprovacaoMatriz){
                $this->aprovacaoCardapioMatrizService->aprovarCard($register, $dataToSave);

                if($isAprovedFilial){
                    $this->aprovacaoCardapioMatrizService->aprovarCardAprovadosFilial($register, $dataToSave);
                } else if($isExpiredDataLimiteAprovacaoFilial){
                    $this->aprovacaoCardapioMatrizService->aprovarCardExpiradosFilial($register, $dataToSave);
                }
            }
            
            if($isExpiredDataLimiteAprovacaoMatriz)
                $responseMessage['expiredLimit'] = $isExpiredDataLimiteAprovacaoMatriz;
            
            if($isRestricted)
                $responseMessage['restrictedCard'] = $isRestricted;
        }
        
        $response->addDataSet(new DataSet('responseData', array($responseMessage)));
    }


    /**
    * Função responsável por aprovar cardápio caso todas as verificações forem
    * atendidas. Aprova cardápio de acordo com a aprovação pela filial.
    * 
    * @param {Request} $request Requisição feita pelo Usuário.
    * @param {Response} $response Resposta a requisição do usuário.
    */
    public function cancelarCard(DTO\Request $request, DTO\Response $response){
        $rows = $request->getRow()->get('ROWS')['dataset'];
        $dataToSave = $request->getRow()->get('DATA');
        $responseMessage = array();

        foreach ($rows as $register) {
            
            $isExpiredDataLimiteAprovacaoMatriz = $this->isExpiredDataLimiteAprovacaoMatriz($register, '01/'.$dataToSave['DTMESCARD']);

            if(!$isExpiredDataLimiteAprovacaoMatriz){
                $this->aprovacaoCardapioMatrizService->cancelarCard($register, $dataToSave);
            }

            $responseMessage['expiredLimit'] = $isExpiredDataLimiteAprovacaoMatriz;
        }
        
        $response->addDataSet(new DataSet('responseData', array($responseMessage)));
    }
    
    
    /**
    * Função responsável por verificar se cardápio possui restrições 
    * contratuais.
    * 
    * @param {Array} $register Registro atual para verificar.
    * @param {String} $mesCard Mês do cardápio a verificar.
    */
    private function isRestrictedCard($register, $mesCard){
        
        $cdFilial = $register['CDFILIAL'];
        $cdServFili = $register['CDSERVFILI'];
        $isRestricted = false;

        if($register['IDRESTRAPROVCARD']=='S'){
            $isRestricted = $this->aprovacaoCardapioMatrizService->isRestrictedCard($cdFilial, $cdServFili, $mesCard);
        } 
        
        return $isRestricted;
    }
    
    
    /**
    * Função responsável por verificar se cardápio já foi aprovado
    * 
    * @param {Array} $register Registro atual para verificar.
    * @param {String} $mesCard Mes atual para verificar.
    */
    private function isAprovedCardFilial($register, $mesCard){
        
        $cdFilial = $register['CDFILIAL'];
        $cdServFili = $register['CDSERVFILI'];
        
        return $this->aprovacaoCardapioMatrizService->isAprovedCardFilial($cdFilial, $cdServFili, $mesCard);
    }


    /**
    * Função responsável por verificar se data limite para aprovação
    * de cardápio na filial está expirada.
    * 
    * @param {Array} $register Registro atual para verificar.
    * @param {String} $mesCard Mes atual para verificar.
    */
    private function isExpiredDataLimiteAprovacaoFilial($register, $mesCard){
        
        $cdFilial = $register['CDFILIAL'];
        return $this->aprovacaoCardapioMatrizService->isExpiredDataLimiteAprovacaoFilial($cdFilial, $mesCard);
    }
    
    
    /**
    * Função responsável por verificar se data limite para aprovação
    * de cardápio na filial está expirada.
    * 
    * @param {Array} $register Registro atual para verificar.
    * @param {String} $mesCard Mes atual para verificar.
    */
    private function isExpiredDataLimiteAprovacaoMatriz($register, $mesCard){
        
        $cdFilial = $register['CDFILIAL'];
        return $this->aprovacaoCardapioMatrizService->isExpiredDataLimiteAprovacaoMatriz($cdFilial, $mesCard);
    }
    
    
    /**
    * Função responsável por salvar o campo de observação geral caso for
    * alterado.
    * 
    * @param {Request} $request Requisição feita pelo Usuário.
    * @param {Response} $response Resposta a requisição do usuário.
    */
    public function saveObservGer(DTO\Request $request, DTO\Response $response){
        try{
            $obsGer = $request->getRow()->get('DSGERAPROCARDMATR');
            $dtMesCard = '01/'.$request->getRow()->get('DTMESCARD');
    
            if(strlen($obsGer)<=2000){
                $this->aprovacaoCardapioMatrizService->saveObservGer($obsGer, $dtMesCard);
            } else {
                throw new \Exception("Observação geral maior do que aceito pelo banco de dados.");
            }
        } catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
        
    }

    /**
    * Função responsável por pegar os dados do campo VRPUCTRLAPRCARDMATR
    * no banco de dados.
    *
    * @param {Array[Object]} $dataSet Conjunto de cardapios para retorno. 
    * @param {Array[Object]} $dataToExecute Dados para geração do nrConsplan. 
    *
    * @return {Array} Array de dados de valor dos registros passados por 
    *   parâmetro.
    */
    private function getVrpuCtrlAprCardMatr($dataSet, $dataToExecute){
        
        $nrLista = $this->aprovacaoCardapioMatrizService->getNrLista();

        $mesCard = $dataToExecute['DTMESCARD'];
        $newDate = DateTime::createFromFormat('d/m/Y', '01/'.$mesCard)->format('Y-m-d');
        $firstDay = '01/'.$mesCard;
        $lastDay = date('t', strtotime($newDate)).'/'.$mesCard;

        $dataToExecute['CDOPERADOR'] = $this->environment->getCdOperador();
        $dataToExecute['DT_INICIAL'] = $firstDay;
        $dataToExecute['DT_FINAL'] = $lastDay;

        $this->recordArrayOnList($dataSet, $nrLista);
        $nrConsPlan = $this->aprovacaoCardapioMatrizService->getConsPlanNumber($nrLista, $dataToExecute)['P_NRCONSPLAN'];
        $this->aprovacaoCardapioMatrizService->removeLista($nrLista);

        return $this->aprovacaoCardapioMatrizService->getVrpuCtrlAprCardMatr($nrConsPlan, $dataToExecute);

    }

    /**
    * Função responsável por gravar array na tabela temporária de lista
    * no banco de dados.
    *
    * @param {Array} $dataSet Array de dados a serem gravados.
    * @param {Array} $nrLista Numero da lista onde será gravado.
    */
    private function recordArrayOnList($dataSet, $nrLista){
        foreach($dataSet as $data){

            $regLista = $data['CDFILIAL'].$data['CDSERVFILI'];
            $this->aprovacaoCardapioMatrizService->insertLista($regLista, $nrLista);

        }
    }
}